from logos_cdi.abstract import AbstractContainer, AbstractResource
from logos_cdi.container import Container, ContainerGroup


class ResourceBuilder:

    def __init__(self):
        self.type = None
        self.parameters = {}
        self.arguments = []

    def from_type(self, type):
        self.type = type
        return self

    def with_parameters(self, **kwargs):
        self.parameters = kwargs
        return self

    def with_arguments(self, *args):
        self.arguments = args
        return self

    def build(self, container: AbstractContainer) -> AbstractResource:
        klzz = container.get(f'resource:{self.type}')
        return klzz(*self.arguments, **self.parameters)


class ContainerBuilder:

    def __init__(self):
        self.containers = []
        self.resources = {}

    def add_container(self, container: AbstractContainer):
        self.containers.append(container)
        return self

    def add_resource(self, name: str, type: str, *args, **kwargs):
        self.resources[name] = {
            "type": type,
            "arguments": args,
            "parameters": kwargs
        }
        return self

    def build(self, container: AbstractContainer) -> AbstractContainer:
        container = ContainerGroup([
            Container({
                key: ResourceBuilder()
                    .from_type(value['type'])
                    .with_arguments(*value['arguments'])
                    .with_parameters(**value['parameters'])
                    .build(container)
                for key, value in self.resources.items()
            }),
            *self.containers
        ])
        return container



from abc import ABC, abstractmethod
from argparse import ArgumentParser
from typing import Dict

from logos_cdi.abstract import AbstractContainer
from logos_cdi.application import Module
from sys import argv


class AbstractCommand(ABC):

    __singleton__ = ['application']

    def __init__(self, argument_parser: ArgumentParser):
        self.argument_parser = argument_parser
        self._arguments = None

    def define_arguments(self, argument_parser: ArgumentParser):
        pass

    @property
    def arguments(self):
        if self._arguments is None:
            self.define_arguments(self.argument_parser)
            self._arguments = self.argument_parser.parse_known_args(getattr(self, 'argv', None))[0]
        return self._arguments

    @abstractmethod
    async def execute(self):
        raise NotImplementedError('please implement this method')


class CommandDelegator(AbstractCommand):

    def __init__(self, argument_parser: ArgumentParser, commands: Dict[str, AbstractCommand]):
        super().__init__(argument_parser)
        self.commands = commands

    def define_arguments(self, argument_parser: ArgumentParser):
        subparsers = argument_parser.add_subparsers(title='command', help='command to execute', required=True, dest='command')
        for command_name, command in self.commands.items():
            command_parser = subparsers.add_parser(command_name)
            command.define_arguments(command_parser)

    async def execute(self):
        command = self.commands[self.arguments.command]
        command.argv = argv[2::]
        await command.execute()


class CommandMiddleware(AbstractCommand):

    def __init__(self, argument_parser: ArgumentParser, context: AbstractContainer):
        super().__init__(argument_parser)
        self.context = context

    async def execute(self):
        async with self.context.get('command_middleware_manager') as context:
            command_delegator = context.get('command_delegator')
            await command_delegator.execute()


__container__ = Module()
__container__.container_builder()\
    .add_resource('commands', 'group', regex=r'^command:(?P<name>[\w+:]*)', resolve_resources=True)\
    .add_resource('argument_parser', 'service', factory='class::argparse:ArgumentParser')\
    .add_resource(
        name='command_middleware_manager',
        type='service',
        factory='class::logos_cdi.middleware:MiddlewareContextManager',
        parameters={'context': '%context%', 'config_obj': '%application%', 'config_path': 'command["middleware"]'}
    )\
    .add_resource(
        name='command_delegator',
        type='service',
        factory='class::logos_cdi.command:CommandDelegator',
        parameters={"argument_parser": "%argument_parser%", "commands": "%commands%"}
    )\
    .add_resource(
        name='command',
        type='service',
        factory='class::logos_cdi.command:CommandMiddleware',
        parameters={"argument_parser": "%argument_parser%", "context": "%context%"}
    )



from asyncio import run
from logos_cdi.application import Application


def main():
    app = Application(
        modules=[
            'logos_cdi',
            'logos_cdi.manager',
            'logos_cdi.command'
        ]
    )

    command = app.get('command')
    run(command.execute())


if __name__ == '__main__':
    main()



from __future__ import annotations
from abc import ABC, abstractmethod
from typing import TypeVar, Type

R = TypeVar('R')


class AbstractContainer(ABC):
    """Abstract Container Class"""

    @abstractmethod
    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        raise NotImplementedError('Please implement this method')

    @abstractmethod
    def has(self, name: str, context: AbstractContainer = None) -> bool:
        raise NotImplementedError('Please implement this method')

    def resource_names(self) -> list:
        raise NotImplementedError('Please implement this method')


class AbstractResource(ABC):
    """Abstract Resource Class"""

    @abstractmethod
    def resolve(self, container: AbstractContainer, _name: str = None):
        raise NotImplementedError('Please implement this method')


class AbstractContextManager(ABC):

    def __init__(self, context: AbstractContainer):
        self.context = context

    @abstractmethod
    async def __aenter__(self) -> AbstractContainer:
        raise NotImplementedError('Please implement this method')

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        raise NotImplementedError('Please implement this method')


from logos_cdi.application import Module
from logos_cdi.container import Container
from logos_cdi.resource import Class, Group, Parameter

__container__ = Container({
    'resources': Group(r'^resource:(?P<name>\w+)'),
    'resource:class': Class('logos_cdi.resource:Class'),
    'resource:service': Class('logos_cdi.resource:Service'),
    'resource:parameter': Class('logos_cdi.resource:Parameter'),
    'resource:group': Class('logos_cdi.resource:Group')
})



from importlib import import_module
from re import compile

from logos_cdi.abstract import AbstractResource, AbstractContainer
from os import environ

from logos_cdi.builder import ResourceBuilder


class Parameter(AbstractResource):

    def __init__(self, value):
        self.value = value

    @classmethod
    def recursive_resolve(cls, value, container: AbstractContainer, **kwargs):
        if isinstance(value, dict):
            return {k: cls.recursive_resolve(v, container, **kwargs) for k, v in value.items()}
        if isinstance(value, list):
            return [cls.recursive_resolve(v, container, **kwargs) for v in value]
        if isinstance(value, str):
            if value.startswith('%') and value.endswith('%'):
                return cls.recursive_resolve(container.get(value[1:-1], container), container, **kwargs)
            if value.startswith('@') and value.endswith('@'):
                return cls.recursive_resolve(environ.get(value[1:-1]), container, **kwargs)
            if value.startswith('__') and value.endswith('__'):
                return cls.recursive_resolve(kwargs.get(value[2: -2]), container, **kwargs)
        return value

    def resolve(self, container: AbstractContainer, _name: str = None):
        return self.recursive_resolve(self.value, container, name=_name)


class Class(AbstractResource):

    def __init__(self, class_path: str):
        self.module_path, self.class_name = class_path.split(':', 1)

    def resolve(self, container: AbstractContainer, _name: str = None):
        return getattr(import_module(self.module_path), self.class_name)


class Service(AbstractResource):

    def __init__(self, factory: str, parameters: dict = None, singleton: list = None):
        self.factory = factory
        self.parameters = parameters or {}
        self.singleton = singleton

    def resolve(self, container: AbstractContainer, _name: str = None):
        factory = None
        if self.factory.startswith('class'):
            factory = lambda parameters: ResourceBuilder()\
                    .from_type('class')\
                    .with_arguments(self.factory.replace('class::', '', 1))\
                    .build(container)\
                    .resolve(container, _name).__call__(**parameters)
        else:
            factory = lambda parameters: container.get(self.factory).new(**parameters)

        resource = factory(ResourceBuilder()
                       .from_type('parameter')
                       .with_arguments(self.parameters)
                       .build(container).resolve(container, _name)
        )

        if self.singleton is not None:
            setattr(resource, '__singleton__', self.singleton)

        return resource


class Group(AbstractResource):

    def __init__(self, regex: str, resolve_resources=False):
        self.regex = compile(regex)
        self.resolve_resources = resolve_resources

    def resolve(self, container: AbstractContainer, _name: str = None):
        groups = {}
        for resource_name in container.resource_names():
            match = self.regex.match(resource_name)
            if match:
                if self.resolve_resources:
                    groups[match.group('name')] = ResourceBuilder()\
                            .from_type('parameter')\
                            .with_arguments(f'%{resource_name}%')\
                            .build(container)\
                            .resolve(container, resource_name)
                else:
                    groups[match.group('name')] = resource_name
        return groups










from logos_cdi.abstract import AbstractContextManager, AbstractContainer
from property_accessor import PropertyAccessor


class MiddlewareContextManager(AbstractContextManager):

    def __init__(self, context: AbstractContainer, config_obj: object, config_path: str):
        super().__init__(context)
        self.middlewares_config = PropertyAccessor(config_obj).get(config_path, [])
        self.context_managers = {}

    async def __aenter__(self) -> AbstractContainer:
        context = self.context
        for context_manager_service in self.middlewares_config:
            self.context_managers[context_manager_service] = context.get(context_manager_service)
            context = await self.context_managers[context_manager_service].__aenter__()
        return context

    async def __aexit__(self, *args, **kwargs):
        for context_manager in self.context_managers.values():
            await context_manager.__aexit__(*args, **kwargs)



from sys import path, argv
from os.path import abspath
from argparse import ArgumentParser
from importlib import import_module
from logos_cdi import Module
from logos_cdi.abstract import AbstractContainer
from logos_cdi.application import Application
from logos_cdi.command import AbstractCommand

path.append(abspath('.'))

try:
    app = getattr(import_module('main'), 'app')
except:
    app = None


class ManagerCommand(AbstractCommand):

    def __init__(self, argument_parser: ArgumentParser, context: AbstractContainer):
        super().__init__(argument_parser)
        self.context = context

    def define_arguments(self, argument_parser: ArgumentParser):
        argument_parser.add_argument('--application', required=app is None)

    async def execute(self):
        global app
        if not app:
            module_path, app_var = self.arguments.application.split(':')
            try:
                app = getattr(import_module(module_path), app_var, None)
            except:
                raise ValueError(f'"{self.arguments.application}" is invalid application url')
        if not isinstance(app, Application):
            raise ValueError(f'"{self.arguments.application or "main:app"}" not is Application instance')
        command = app.get('command')
        await command.execute()


__container__ = Module()
__container__.container_builder()\
    .add_resource(
        name='command',
        type='service',
        factory='class::logos_cdi.manager:ManagerCommand',
        parameters={"argument_parser": ArgumentParser(add_help=False), "context": '%context%'}
    )



from importlib import import_module
from zipimport import zipimporter
from typing import Callable, Type

from logos_cdi.abstract import AbstractContainer, R
from logos_cdi.builder import ContainerBuilder
from logos_cdi.container import CacheContainer, ContainerGroup, Container
from logos_cdi.resource import Parameter


class Module(AbstractContainer):

    def __init__(self):
        self._container = None
        self._container_builder = ContainerBuilder()

    def container_builder(self):
        return self._container_builder

    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        if not self._container:
            self._container = self._container_builder.build(context)
        return self._container.get(name, context, _type)

    def has(self, name: str, context: AbstractContainer = None) -> bool:
        if not self._container:
            self._container = self._container_builder.build(context)
        return self._container.has(name, context)

    def resource_names(self) -> list:
        container = self._container
        if not container:
            from logos_cdi import __container__
            container = self._container_builder.build(__container__)
        return container.resource_names()


class Application(AbstractContainer):

    def __init__(self, modules, **kwargs):
        self.modules = modules
        for field, value in kwargs.items():
            setattr(self, field, value)
        self._container = None

    @property
    def container(self):
        if not self._container:
            containers = [
                Container({'application': Parameter(self)})
            ]
            for module_path in self.modules: #type: str
                if module_path.startswith('zip://'):
                    zip_name, path = module_path.replace('zip://', '').split(':')
                    zip_module = zipimporter(f'{zip_name}.zip')
                    module = zip_module.load_module(path)
                    if hasattr(module, '__container__'):
                        containers.append(getattr(module, '__container__'))
                else:
                    module = import_module(module_path)
                    if hasattr(module, '__container__'):
                        containers.append(getattr(module, '__container__'))
            self._container = CacheContainer('application', ContainerGroup(containers))
        return CacheContainer('runtime', self._container)

    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        return self.container.get(name, context or self, _type)

    def has(self, name: str, context: AbstractContainer = None) -> bool:
        return self.container.has(name, context or self)

    def resource_names(self) -> list:
        return self.container.resource_names()





from typing import Dict, Type, List

from .abstract import AbstractContainer, AbstractResource, R
from .errors import ResourceNotFound


class Container(AbstractContainer):

    def __init__(self, resources: Dict[str, AbstractResource]):
        self.resources = resources

    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        if name == 'context':
            return context or self
        resource = self.resources.get(name)
        if not resource:
            raise ResourceNotFound(context or self, name)
        return resource.resolve(context or self, name)

    def has(self, name: str, context: AbstractContainer = None) -> bool:
        return name == 'context' or name in self.resources.keys()

    def resource_names(self) -> list:
        return ['context', *self.resources.keys()]


class ContainerGroup(AbstractContainer):

    def __init__(self, containers: List[AbstractContainer]):
        self.containers = containers
        self._cache = {}
        self._resource_names_cache = {}

    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        if name not in self._cache.keys():
            for container in self.containers:
                if container.has(name, context or self):
                    self._cache[name] = lambda: container.get(name, context or self, _type)
                    break
            else:
                raise ResourceNotFound(context or self, name)
        return self._cache[name]()

    def has(self, name: str, context: AbstractContainer = None) -> bool:
        return name in self._cache.keys() or any(container.has(name, context or self) for container in self.containers)

    def resource_names(self) -> list:
        if not self._resource_names_cache:
            self._resource_names_cache = list(dict.fromkeys([item for container in self.containers for item in container.resource_names()]))
        return self._resource_names_cache


class CacheContainer(AbstractContainer):

    def __init__(self, context_name: str, container: AbstractContainer):
        self.container = container
        self.cache = {}
        self.context_name = context_name

    def get(self, name: str, context: AbstractContainer = None, _type: Type[R] = object) -> R:
        if name == 'scope':
            return self.context_name
        if name not in self.cache.keys():
            resource = self.container.get(name, context or self, _type)
            if self.context_name in getattr(resource, '__singleton__', ['runtime']):
                self.cache[name] = resource
            else:
                return resource
        return self.cache[name]

    def has(self, name: str, context: AbstractContainer = None) -> bool:
        return name == 'scope' or self.container.has(name, context or self)

    def resource_names(self) -> list:
        return ['scope', *self.container.resource_names()]



from logos_cdi.abstract import AbstractContainer


class LogosCDIError(Exception): pass


class ResourceNotFound(LogosCDIError):

    def __init__(self, container: AbstractContainer, service_name: str):
        self.container = container
        self.service_name = service_name
        super(ResourceNotFound, self).__init__(f'service "{service_name}" not found')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logos_cdi']

package_data = \
{'': ['*']}

install_requires = \
['property-accessor>=0.1.0,<0.2.0']

entry_points = \
{'console_scripts': ['logos = logos_cdi.__main__:main']}

setup_kwargs = {
    'name': 'logos-cdi',
    'version': '0.3.1',
    'description': '',
    'long_description': "![logo](https://raw.githubusercontent.com/elieltonkremer/logos_cdi/main/logo.svg)\n\n[![Upload Python Package](https://github.com/elieltonkremer/logos_cdi/actions/workflows/python-publish.yml/badge.svg)](https://github.com/elieltonkremer/logos_cdi/actions/workflows/python-publish.yml)\n\nSimple and powerful python container dependency injection module\n\n\n## installation\n\n```bash\n> poetry add logos_cdi\n```\n\n## usage\n\ncreate `main.py` file and create `logos_cdi.application:Application` instance with a modules for usage.\n\n```py\n\nfrom logos_cdi.application import Application\n\n\napp = Application([\n    'logos_cdi',\n    'logos_cdi.command'\n])\n\n```\n\nin your terminal with venv actived type `logos -h` command and press enter.\n\n```\n\n> logos -h\nusage: logos [-h] {} ...\n\noptions:\n  -h, --help  show this help message and exit\n\ncommand:\n  {}          command to execute\n\n```\n\nthis is your app's command manager all your commands you can see here\n\nPS. your commands are loaded from the modules used in the application, you can implement them, see `./logos_cdi/command.py` file to understand how to create a module.\n",
    'author': 'Elielton Kremer',
    'author_email': 'elielton@integra.do',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
